# coding: UTF-8
'''
Created on May 4, 2010

@author: hernan
@note: Version de documentacion hasta revision 1715
'''

from ErrorCalleSinAlturas import ErrorCalleSinAlturas

class Calle:
    '''
    @ivar cod: Codigo de calle
    @type cod: Integer 
    @ivar nom: Nombre oficial de la calle
    @type nom: String
    @ivar alturas: [optional] Array conteniendo los rangos de altura validos para esta calle
    @type alturas: String [Array]
    @ivar cruces:  [optional] Array conteniendo los id de las calles que se cruzan con esta
    @type cruces: Integer [Array]
    '''
    cod = 0
    nom = ""
    alturas = []
    cruces = []

    def __init__(self, cod, nom, alturas = [], cruces = []):
        '''
        @param cod: Codigo de calle
        @type cod: Integer 
        @param nom: Nombre oficial de la calle
        @type nom: String
        @param alturas: [optional] Array conteniendo los rangos de altura validos para esta calle
        @type alturas: String [Array]
        @param cruces:  [optional] Array conteniendo los id de las calles que se cruzan con esta
        @type cruces: Integer [Array]
        '''
        self.cod = cod
        self.nom = nom
        self.alturas = alturas
        self.cruces = cruces

    def __str__(self):
        retval = "- CLASE: Calle\n"
        retval += "\tcod: "+str(self.cod)+"\n"
        retval += "\tnom: "+self.nom+"\n"
        retval += "\talturas: "+self.alturas.__str__()+"\n"
        retval += "\tcruces: "+self.cruces.__str__()+"\n"
        return retval
    
    def alturaValida(self, alt):
        '''
        Verifica si la altura es valida para esta calle. En caso de tratarse de una calle sin alturas oficiales
        asiganadas se lanza la excepcion <code>ErrorCalleSinAlturas</code>. 
        @param alt: Altura a validar
        @type alt: Integer
        @return: True si la altura es valida para esta calle
        @rtype: Boolean
        '''
        if(isinstance(self.alturas, list)):
            if (len(self.alturas) == 0):
                raise ErrorCalleSinAlturas(self.nom)
                return False
            valid = False
            if(alt.isdigit()):
                alt = int(alt)
            else:
                alt = 0
            for a in self.alturas:
                valid = valid or ((int(a[0]) <= alt) and (int(a[1]) >= alt))
                if valid:
                    break
            return valid

    def getTramos(self):
        '''
        Devuelve la lista de tramos de alturas validas para esta calle
        @return: Listado de tramos de alturas validas para esta calle
        @rtype: Array
        '''
        return self.alturas

    def toString(self):
        '''
        Devuelve un string con el nombre de la calle 
        @return: Nombre de la calle
        @rtype: String
        '''
        return self.nom

    def seCruzaCon(self, calle):
        '''
        Verifica si la calle (instancia de la clase usig.Calle) recibida como parametro se cruza con esta 
        @param calle: Calle a verificar si se intersecta con esta
        @type calle: Calle
        @return: True en caso de que exista el cruce correspondiente
        @rtype: Boolean
        '''
        if(len(self.cruces) > 0):
            return(calle.cod in self.cruces)
