# coding: UTF-8
'''
Created on May 5, 2010

@author: hernan
@note: Version de documentacion hasta revision 1715
'''
from Calle import Calle
import re
from Punto import Punto

class Direccion:
    '''
    @ivar calle: Calle de la direccion
    @type calle: Calle 
    @ivar calleCruce: Calle con la que se cruza
    @type calleCruce: Calle 
    @ivar altura: Altura de la calle
    @type altura: Integer
    @ivar tipo:  tipo de la calle
    @type tipo: {CALLE_ALTURA = 0, CALLE_Y_CALLE = 1}   
    @ivar smp: Seccion-Manzana-Parcela
    @type smp: String
    @ivar coordenadas: Geocodificacion
    '''
    CALLE_ALTURA = 0
    CALLE_Y_CALLE = 1
    
    calle = None
    calleCruce = None
    altura = 0
    tipo = None
    smp = ''
    coordenadas = None 

    def __init__(self, calle1, calle2OAltura):
        '''
        @param calle1: Instancia de la clase Calle
        @type calle1: Calle
        @param calle2OAltura: Instancia de Calle que se cruza con calle1 o altura correspondiente de calle1
        @type calle2OAltura: Calle o Integer
        '''
        if(isinstance(calle1, Calle)):
            self.calle = calle1
        else:
            return None
    
        if(isinstance(calle2OAltura, Calle)):
            self.calleCruce = calle2OAltura
            self.tipo = self.CALLE_Y_CALLE;
        else:
            try:
                self.altura = int(calle2OAltura)
                self.tipo = self.CALLE_ALTURA
            except ValueError:
                return None

    def __str__(self):
        retval = "- CLASE: Direccion\n"
        retval += "\tcalle: "+self.calle.__str__()+"\n"
        retval += "\tcalleCruce: "+self.calleCruce.__str__()+"\n"
        retval += "\taltura: "+str(self.altura)+"\n"
        retval += "\ttipo: "+self.tipo.__str__()+"\n"
        retval += "\tsmp: "+self.smp+"\n"
#        retval += "\tcoordenadas: "+self.coordenadas__str__()+"\n"
        return retval

    def getCalle(self):
        '''
        Devuelve la instancia de Calle correspondiente 
        @return: instancia de Calle
        @rtype: Calle
        '''
        return self.calle

    def getCalleCruce(self):
        '''
        Devuelve la instancia de Calle correspondiente a la interseccion 
        @return: instancia de Calle correspondiente a la interseccion
        @rtype: Calle
        '''
        if(self.tipo == self.CALLE_Y_CALLE):
            return self.calleCruce
        else:
            return None

    def getAltura(self):
        '''
        Devuelve la altura correspondiente 
        @return: la altura correspondiente
        @rtype: Integer
        '''
        return self.altura

    def getTipo(self):
        '''
        Devuelve el tipo de direccion correspondiente
        @return: tipo de direccion correspondiente
        @rtype: {CALLE_ALTURA = 0, CALLE_Y_CALLE = 1}
        '''
        return self.tipo
        
        
    def toString(self):
        '''
        Devuelve un string con la direccion escrita correctamente para mostrar
        @return: Direccion como texto
        @rtype: String
        '''
        if(self.tipo == self.CALLE_ALTURA):
            if(self.altura>0):
                altura = str(self.altura)
            else:
                altura = 'S/N'
            retval = self.calle.toString()+' '+altura            
        else:
            if(re.match('(?i)(I|Hi|HI)', self.calleCruce.toString()) != None):
                separador = ' e '
            else:
                separador = ' y '
            retval = self.calle.toString()+separador+self.calleCruce.toString() 
        return retval

    def setCoordenadas(self, pt):
        '''
        Setea las coordenadas de la geocodificacion de esta direccion
        @param pt: Instancia de usig.Punto que contiene las coordenadas
        @type pt: Punto
        '''
        self.coordenadas = Punto(pt.getX(), pt.getY())

    def setSMP(self, SMP):
        '''
        Setea el codigo de seccion-manzana-parcela correspondiente a esta direccion si existiera
        @param SMP: Codigo de seccion-manzana-parcela
        @type SMP: String
        '''
        self.smp = SMP

    def getCoordenadas(self):
        '''
        Devuelve las coordenadas asociadas a esta direccion en caso de estar disponibles o null en caso contrario
        @return: Instancia de usig.Punto con las coordenadas correspondientes
        @rtype: Punto
        '''
        return self.coordenadas

    def getSMP(self):
        '''
        Devuelve el codigo de seccion-manzana-parcela asociado a esta direccion en caso de estar disponible o '' en caso contrario
        @return: Codigo de seccion-manzana-parcela
        @rtype: String
        '''
        return self.smp
