# coding: UTF-8
'''
Created on Apr 28, 2010

@author: hernan
@note: Version de documentacion hasta revision 1715
'''

class ErrorCalleInexistenteAEsaAltura(Exception):
    '''
    Excepcion para altura inexistente para una calle con altura.
    '''
    calle = ""
    matchings = []
    altura = 0

    def __init__(self, calle, matchings, altura):
        '''
        Constructor
        @param calle: Nombre oficial de la calle
        @type calle: String
        @param matchings: Array de instancias de Calle que matchean el string 'calle'
        @type matchings: Array de Calle
        @param altura: Altura invalida de la calle
        @type altura: Integer
        '''
        self.calle = calle
        self.matchings = matchings
        self.altura = altura
    
    def __str__(self):
        '''
        Devuelve un mensaje de error con el nombre de la calle
        @return: Mensaje de error
        @rtype: String
        '''
        return "La calle "+self.calle+" no existe a la altura "+self.altura 

    def getCalle(self):
        '''
        Devuelve el nombre de la calle 
        @return: Nombre de la calle
        @rtype:  String
        '''
        return self.calle

    def getMatchings(self):
        '''
        Devuelve el array de matchings para la calle 
        @return: Instancias de Calle que matchean 'calle'
        @rtype: Array de Calle
        '''
        return self.matchings

    def getAltura(self):
        '''
        Devuelve la altura invalida de la calle 
        @return: Altura
        @rtype: Integer
        '''
        return self.altura

    def getErrorMessage(self):
        '''
        Devuelve un mensaje de error mas descriptivo y amigable
        @return: Mensaje de error
        @rtype: String
        '''
        msg = 'La altura indicada no es v&aacute;lida para la calle ingresada. A continuaci&oacute;n se muestran algunas opciones v&aacute;lidas halladas:<ul>'
        for calle in self.matchings:
            tramos = calle.getTramos()
            for tramo in tramos:
                msg+='<li>'+calle.nombre+' '+tramo[0]+'-'+tramo[1]+'</li>'
        msg+='</ul>'            
        return msg