# coding: UTF-8
'''
Created on Apr 29, 2010

@author: hernan
@note: Version de documentacion hasta revision 1717
'''

from StringDireccion import StringDireccion
from Callejero import Callejero
from Errors import *
from Direccion import Direccion
import copy

class NormalizadorDirecciones:
    '''
    NormalizadorDirecciones
    Esta clase implementa integramente un normalizador de direcciones que utiliza el Callejero de USIG para transformar un string en una direccion normalizada.
    @ivar aceptarCallesSinAlturas: Indica al normalizador si debe permitir como altura S/N para las calles sin numero. Por defecto es False. Ej: de los italianos S/N
    @type aceptarCallesSinAlturas: Boolean  
    '''
    aceptarCallesSinAlturas = False 
    c = None
    
    def __init__(self, aceptarCallesSinAlturas = False):
        '''
        @param aceptarCallesSinAlturas: Indica al normalizador si debe permitir como altura S/N para las calles sin numero. Por defecto es False. Ej: de los italianos S/N
        @type aceptarCallesSinAlturas: Boolean  
        '''
        self.aceptarCallesSinAlturas = aceptarCallesSinAlturas
        try:
            self.c = Callejero()
        except Exception,  e:
            raise e
        
    def normalizar(self, dir, maxOptions = 10):
        ''' 
        @param dir: La cadena a ser transformada en direccion
        @type dir: String
        @param maxOptions: Maximo numero de opciones a retornar. Por defecto es 10.
        @type maxOptions: Integer
        @return: Las opciones halladas que se corresponden con dir
        @rtype: Array de Direccion
        '''
        dir = dir.upper()
        if type(dir) != unicode:
            dir = unicode(dir, 'utf-8')
            
        strDir = StringDireccion(dir, self.aceptarCallesSinAlturas)
        
        res = []
        if strDir.tipo == strDir.CALLE:
            res = self.c.matcheaCalle(strDir.strInput, maxOptions)
        elif strDir.tipo == strDir.CALLE_ALTURA:
            res = self.normalizarCalleAltura(strDir, maxOptions)
        elif strDir.tipo == strDir.CALLE_Y_CALLE:
            res = self.normalizarCalleYCalle(strDir, maxOptions)
            if len(res) == 0:
                strDir.setearCalleAltura()
                res = self.normalizarCalleAltura(strDir, maxOptions)
        elif strDir.tipo == strDir.INVALIDO:
            res = []

        if isinstance(res, list):
            if(len(res) > 0):
                return res
            else:
                raise ErrorCalleInexistente(strDir)
        else:
            return res

    def normalizarCalleAltura(self, strDir, maxOptions):
        '''
        Normaliza una direccion de tipo Calle-altura
        @param strDir: La direccion a ser normalizada
        @type strDir: StringDireccion
        @param maxOptions: Maximo numero de opciones a retornar.
        @type maxOptions: Integer
        @return: Las opciones halladas que se corresponden con dir
        @rtype: Array
        '''
        calles = self.c.matcheaCalle(strDir.strCalles)
        try:
            opts = self.validarAlturas(strDir, calles, maxOptions)
        except ErrorCalleSinAlturas, e:
            raise e
        if(len(opts) == 0 and len(calles) > 0):
            strDir.quitarAvsCalle()
            calles = self.c.matcheaCalle(strDir.strCalles)
            try:
                opts = self.validarAlturas(strDir, calles, maxOptions)
            except Exception,  e:
                raise e
            opts = self.filtrarCallesQueNoSonAv(opts)
            if(len(opts) == 0):
                raise ErrorCalleInexistenteAEsaAltura(strDir.strCalles, calles, strDir.strAltura)
        elif(len(opts) == 0 and len(calles) == 0):
            strDir.quitarPasajes()
            calles = self.c.matcheaCalle(strDir.strCalles)
            try:
                opts = self.validarAlturas(strDir, calles, maxOptions)
            except Exception,  e:
                raise e
        return opts

    def validarAlturas(self, strDir, calles, maxOptions):
        '''
        Verifica si la altura es valida para las calles matcheadas
        @param strDir: La direccion a ser verificada
        @type strDir: StringDireccion
        @param calles: Calles que matchean con strDir
        @type calles: Array de Calle
        @param maxOptions: Maximo numero de opciones a retornar.
        @type maxOptions: Integer
        @return: Las Calles que matchean y que son validas a la altura especificada
        @rtype: Array de Direccion
        '''
        retval = []
        hayCalleSN = False
        for i in range(len(calles)):
            try:
                if(calles[i].alturaValida(strDir.strAltura)):
                    retval.append(Direccion(calles[i],strDir.strAltura))
            except ErrorCalleSinAlturas:
                if(self.aceptarCallesSinAlturas and strDir.esAlturaSN(strDir.strAltura)):
                    retval.append(Direccion(calles[i],0))
                hayCalleSN = True
                    
            if(maxOptions != 0 and len(retval) >= int(maxOptions)):
                break
        if(hayCalleSN and len(retval) == 0):
            raise ErrorCalleSinAlturas(strDir.strInput)
        return retval
        
    def filtrarCallesQueNoSonAv(self, dirs, getFunc='getCalle'):
        '''
        Verifica si las direcciones en dirs tienen tramos como avenidi
        @param dirs: Lista de direcciones a verificar
        @type dirs: Array de Direccion
        @param getFunc: Nombre del metodo para obtener la calle de la Direccion
        @type getFunc: String
        @return: Lista de Calle que tienen tramo como avenida
        @rtype: Array de Direccion
        '''
        opts=[]
        for i in range(len(dirs)):
## Se puede hacer directamente sin usar if?
            if(getFunc=='getCalle'):
                dir = dirs[i].getCalle()
            elif(getFunc=='getCalleCruce'):
                dir = dirs[i].getCalleCruce()
            if(self.c.tieneTramosComoAv(dir)):
                opts.append(dirs[i])
        return opts

    def normalizarCalleYCalle(self, strDir, maxOptions):
        '''
        Normaliza una direccion de tipo calle-calle
        @param strDir: La direccion a ser normalizada
        @type strDir: StringDireccion
        @param maxOptions: Maximo numero de opciones a retornar. Por defecto es 10.
        @type maxOptions: Integer
        @return: Las opciones halladas que se corresponden con dir
        @rtype: Array de Direccion 
        '''
        calles1 = self.c.matcheaCalle(strDir.strCalles[0], maxOptions)
        calles2 = self.c.matcheaCalle(strDir.strCalles[1], maxOptions)
# Armo una lista tabu para evitar agregar 2 veces una interseccion
# de una calle con otra que es calle y avenida a la vez
# Ej. CIUDAD DE LA PAZ y MONROE y CIUDAD DE LA PAZ y MONROE AV.
        matches = []
        opts = []
        for i in range(len(calles1)):
            for j in range(len(calles2)):
                if((calles1[i].cod != calles2[j].cod) and (not self.matchCode(calles1[i], calles2[j]) in matches) and (calles1[i].seCruzaCon(calles2[j]) and calles2[j].seCruzaCon(calles1[i]))):
                    opts.append(Direccion(calles1[i], calles2[j]))
                    matches.append(self.matchCode(calles1[i],calles2[j]))
                    if(len(opts) >= maxOptions):
                        break
            if(len(opts) >= maxOptions):
                break

        if(len(opts) == 0 and len(calles1) > 0 and len(calles2) > 0):
            palabrasCalle1 = strDir.strCalles[0].split(u' ')
            palabrasCalle2 = strDir.strCalles[1].split(u' ')
# Pasarlo a funcion
            if((u'AV' in palabrasCalle1 ) or (u'AVDA' in palabrasCalle1) or (u'AVENIDA' in palabrasCalle1)):
                strDir1 = copy.copy(strDir)
                strDir1.quitarAvsCalle()
                try:
                    opts1 = self.normalizarCalleYCalle(strDir1, maxOptions) 
                except:
                    raise ErrorCruceInexistente(strDir.strCalles[0], calles1, strDir.strCalles[1], calles2)
                self.filtrarCallesQueNoSonAv(opts1)
                if(isinstance(opts1, list)):
                    return opts1
# Pasarlo a funcion
            if((u'AV' in palabrasCalle2 ) or (u'AVDA' in palabrasCalle2) or (u'AVENIDA' in palabrasCalle2)):
                strDir2 = copy.copy(strDir)
                strDir2.quitarAvsCalleCruce()
                try:
                    opts2 = self.normalizarCalleYCalle(strDir2, maxOptions) 
                except:
                    raise ErrorCruceInexistente(strDir.strCalles[0], calles1, strDir.strCalles[1], calles2)
                self.filtrarCallesQueNoSonAv(opts2)
                if(isinstance(opts2, list)):
                    return opts2

# Esto es para salvar el caso de calles con Y en el nombre pero
# que aun no estan escritas completas
# Ej. ORTEGA Y GA
        if(len(opts) < maxOptions):
            calles = self.c.matcheaCalle(strDir.strInput)
            for i in range(len(calles)):
                opts.append(calles[i])
                if(len(opts) >= maxOptions):
                    break
        if(len(opts) == 0 and len(calles1) > 0 and len(calles2) > 0):
            raise ErrorCruceInexistente(strDir.strCalles[0], calles1, strDir.strCalles[1], calles2)
        return opts
    
    def matchCode(self, calle1, calle2):
        return min(calle1.cod, calle2.cod)+max(calle1.cod, calle2.cod)
